/*
  Boxes Rendering Engine
  by David Farrell, fdavid@utdallas.edu
  */

/*
 *  3D Studio object reader
 */

#ifndef __3DSLOAD_H
#define __3DSLOAD_H



#ifdef __cpluplus
	extern "C" {
#endif

//typedef char  byte;
typedef short word;
typedef int   dword;
typedef float float32;




typedef struct {
  word p0,p1,p2;
  word flags;
} H3dsFace;

typedef struct {
  float32 x;
  float32 y;
  float32 z;
} H3dsVert;

typedef struct {
  float32 u;
  float32 v;
} H3dsMap;

// Material structures
typedef struct {
  char name[16];
  int  NumFaces;
  word *faces;
} H3dsMatList;

typedef struct {
  float32 red;
  float32 green;
  float32 blue;
} H3dsRGBColor;

typedef struct {
  char filename[16];
  word options;
  float32 OneOverU;
  float32 OneOverV;
  float32 U;
  float32 V;
} H3dsTexture;

typedef struct {
  char name[16];
  H3dsRGBColor ambient;
  H3dsRGBColor diffuse;
  H3dsRGBColor specular;
  word mat_type;
  H3dsTexture texture;
  int offset;
} H3dsMat;

// Object and scene structures
typedef struct {
  char name[16];          // object name, zero terminated
  int NumFaces;           // number of faces in face list
  int NumVerts;           // number of vertices in vertex list
  int NumMaps;            // number of mapping coordinates in map list
  int matrix;             // 1 if transformation matrix is loaded
  int NumBindings;        // number of binding materials
  H3dsFace *facelist;     // array of face indices
  H3dsVert *vertlist;     // array of vertices
  H3dsMap  *maplist;      // array of texture coordinates
  float32 TraMatrix[3*4]; // 3*3 rotation matrix, 3*1 translation matrix
  H3dsMatList *bindings;  // Binds faces to materials
  int offset;             // Our starting index into outgl's vertex list
} H3dsMeshObj;

typedef struct {
  int meshobjs;              // Mumber of meshobjects in meshobjlist
  H3dsMeshObj *meshobjlist;  // array of meshobjects
  int NumMaterials;          // Number of materials in the scene
  H3dsMat *material;         // array of materials
} H3dsScene;

// Each 3DS data-chunk starts with a 6 byte header.
// The first item in the header is a 2 byte (word) id-number.
// After that follows a dword wich gives the size of
// the data-chunk including the header. The size can be used
// as an relative offset to the next chunk.

enum {
  CHUNK_RGB1      	= 0x0010,  // 3 floats of RGB
  CHUNK_RGB2      	= 0x0011,  // 3 bytes of RGB
  CHUNK_RGB3            = 0x0012,  // What is this?
  CHUNK_MAIN      	= 0x4D4D,
  CHUNK_OBJMESH         = 0x3D3D,
  CHUNK_BKGCOLOR        = 0x1200,
  CHUNK_AMBCOLOR  	= 0x2100,
  CHUNK_OBJBLOCK  	= 0x4000,
  CHUNK_TRIMESH   	= 0x4100,
  CHUNK_VERTLIST        = 0x4110,
  CHUNK_FACELIST        = 0x4120,
  CHUNK_FACEMAT         = 0x4130,
  CHUNK_MAPLIST         = 0x4140,
  CHUNK_SMOOLIST        = 0x4150,
  CHUNK_TRMATRIX        = 0x4160,
  CHUNK_LIGHT     	= 0x4600,
  CHUNK_SPOTLIGHT       = 0x4610,
  CHUNK_CAMERA    	= 0x4700,
  CHUNK_MATERIAL  	= 0xAFFF,
  CHUNK_MATNAME   	= 0xA000,
  CHUNK_AMBIENT   	= 0xA010,
  CHUNK_DIFFUSE   	= 0xA020,
  CHUNK_SPECULAR  	= 0xA030,
  CHUNK_TEXTURE   	= 0xA200,
  CHUNK_BUMPMAP   	= 0xA230,
  CHUNK_MAPFILE   	= 0xA300,
  CHUNK_MAPOPTIONS      = 0xA351,
  CHUNK_KEYFRAMER 	= 0xB000,
  CHUNK_FRAMES          = 0xB008
};

H3dsScene *load3ds( char *filename );
void HFree3dsScene(H3dsScene *scene);

#ifdef __cpluplus
	}
#endif





#endif


