#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "tga.h"

unsigned char *Texture_DecodeTGA(char*, int*, int*, unsigned char* imagebuffer);

unsigned char *Texture_DecodeTGA(char *fname, int *RetWidth, int *RetHeight, unsigned char* imagebuffer)
{
	FILE *f;
	TGA_Header TGAHeader;
	char HybridArea[26];
	unsigned __int8 TGA_Type;
	
	unsigned __int16 XOrigin;
	unsigned __int16 YOrigin;
	unsigned __int16 Width;
	unsigned __int16 Height;
	unsigned __int8 PixelDepth;
	unsigned __int8 Descriptor;
	unsigned __int8 PaintOrder;
	unsigned __int8 AlphaBits;

	int i, j;
	int ImageSize;
	int RealImageSize;
  int bytesPP;
  int pixelOffset;
	unsigned char TempSwap;

  BOOL isBufferMalloced = FALSE;

	printf("TGA Reader loading file '%s'\n", fname);

	f = fopen(fname,"rb");
	if (f == NULL)
	{
		printf("\nError opening file.\n");
		return (NULL);
	}

	fseek(f, -26L, SEEK_END);
	fread(&HybridArea, 26, 1, f);
	if (strncmp("TRUEVISION-XFILE", &HybridArea[8], 16) == 0)	{
    TGA_Type = TGA_NEWTYPE;
    //printf("\nNEW TGA Type\n");
  }
	else {
    TGA_Type = TGA_OLDTYPE;
    //printf("\nOLD TGA Type\n");
  }

	
	fseek(f,0L,SEEK_SET);
	fread(&TGAHeader,sizeof(TGA_Header),1,f);
	
	//printf("\nID len: %d",(int)TGAHeader.IDLen);
	
	//printf("\nColor Map Type: %d",TGAHeader.ColorMapType);
	switch(TGAHeader.ColorMapType)
	{
		case 0:
			//printf("  (No color map data included)");
			break;
		case 1:
			//printf("  (Color map data included)");
			break;
		default:
			//printf(" (Unrecognized)");
			break;
	}

	//printf("\nImage Type: %d",TGAHeader.ImageType);
	switch (TGAHeader.ImageType)
	{
		case 0:
			//printf("   (No image data included)");
			break;
		case 1:
			//printf("   (Uncompressed, color-mapped image)");
			break;
		case 2:
			//printf("   (Uncompressed, true-color image)");
			break;
		case 3:
			//printf("   (Uncompressed, black-and-white image)");
			break;
		case 9:
			//printf("   (RLE encoded, color-mapped image)");
			break;
		case 10:
			//printf("   (RLE encoded, true-color image)");
			break;
		case 11:
			//printf("   (RLE encoded, black-and-white image)");
			break;
		default:
			//printf("   (Unrecognized)");
			break;
	}
	
	if (TGAHeader.ColorMapType!=0)
	{
		//printf("\nColor Map Specification: ");
		// for (i=0; i<5; i++) printf("%d ",(int)TGAHeader.ColorMapSpec[i]);
	  //printf("\nImage Specification: ");
	}

	// Unpack Image Specifications

	XOrigin=*((unsigned __int16 *)&TGAHeader.ImageSpec[0]);
	YOrigin=*((unsigned __int16 *)&TGAHeader.ImageSpec[2]);
	Width=*((unsigned __int16 *)&TGAHeader.ImageSpec[4]);
	Height=*((unsigned __int16 *)&TGAHeader.ImageSpec[6]);
	PixelDepth=*((unsigned __int8 *)&TGAHeader.ImageSpec[8]);
	Descriptor=*((unsigned __int8 *)&TGAHeader.ImageSpec[9]);

	//printf("\nImage X Origin: %d",(int)XOrigin);
	//printf("\nImage Y Origin: %d",(int)YOrigin);
	//printf("\nImage Width: %d",(int)Width);
	//printf("\nImage Height: %d",(int)Height);
	//printf("\nPixel Depth: %d",(int)PixelDepth);

	//printf("\nPaint Order: ");
	switch ((Descriptor & 0x30)>>4)
	{
		case 0:
				PaintOrder = TGA_BOTTOMLEFT;
				//printf("Bottom-Left");
				break;
		case 1:
				PaintOrder = TGA_BOTTOMRIGHT;
				//printf("Bottom-Right");
				break;
		case 2:
				PaintOrder = TGA_TOPLEFT;
				//printf("Top-Left");
				break;
		case 3:
				PaintOrder = TGA_TOPRIGHT;
				//printf("Top-Right");
				break;
		default:
				//printf("Unrecognized");
				break;
	}

	AlphaBits = Descriptor & 0x0f;
	//printf("\nAlpha Bits: %d",(int)AlphaBits);

  /*
  if (AlphaBits == 0)		// WTK RGBA Compatibility
	{
		AlphaBits = 8;
		RealImageSize = (int)Width*(int)Height*((int)PixelDepth/8);
		ImageSize = (int)Width*(int)Height*(((int)PixelDepth+(int)AlphaBits)/8);
	}
	else
	{
		ImageSize = (int)Width*(int)Height*(((int)PixelDepth+(int)AlphaBits)/8);
		RealImageSize = ImageSize;
	}
  */

  ImageSize = (int)Width*(int)Height*(((int)PixelDepth)>>3);
  RealImageSize = ImageSize;

  if (imagebuffer == NULL) {
    isBufferMalloced = TRUE;
    imagebuffer = (unsigned char *) malloc(ImageSize);
  }
  else {
    //printf("\nUsing preallocated buffer @%x", imagebuffer);
  }

  if (imagebuffer != NULL) {
		//if (isBufferMalloced) printf("\n\nAllocated %d bytes for image data\nLoading...", ImageSize);
  }
	else
	{
		printf("\n\nNot enough memory to load image data");
		fclose(f);
		return (NULL);
	}

	
	fseek(f,(long)sizeof(TGA_Header),SEEK_SET);
	fseek(f,(long)TGAHeader.IDLen,SEEK_CUR);
  if (TGAHeader.ColorMapType!=0) {
		printf("\nCouldn't load paletized TGAs");
		if (isBufferMalloced) free(imagebuffer);
		fclose(f);
		return(NULL);
	}

  bytesPP = PixelDepth >> 3;
  //printf("\nBytesPP=%d", bytesPP);
  
	for (i=0; i < RealImageSize;) {
    pixelOffset = i;
    for (j=0; j < bytesPP; j++) {
		  fread(&imagebuffer[i++], 1, 1, f);	// color component
    }
		TempSwap = imagebuffer[pixelOffset];	// Swaps R-B Values
		imagebuffer[pixelOffset] = imagebuffer[pixelOffset+2];
		imagebuffer[pixelOffset+2] = TempSwap;
	}	
  
  //printf("\read=%x/%x", pixelOffset, RealImageSize);

	fclose(f);
		
	//printf("\n");
	
	(*RetWidth) = (int) Width;
	(*RetHeight) = (int) Height;
	return (imagebuffer);
}
